@@ -48,24 +48,24 @@ class docParser(xml.sax.handler.ContentHandler):
 
     def close(self):
         if debug:
-            print "close"
+            print("close")
 
     def getmethodname(self):
         return self._methodname
 
     def data(self, text):
         if debug:
-            print "data %s" % text
+            print("data %s" % text)
         self._data.append(text)
 
     def cdata(self, text):
         if debug:
-            print "data %s" % text
+            print("data %s" % text)
         self._data.append(text)
 
     def start(self, tag, attrs):
         if debug:
-            print "start %s, %s" % (tag, attrs)
+            print("start %s, %s" % (tag, attrs))
         if tag == 'function':
             self._data = []
             self.in_function = 1
@@ -76,11 +76,11 @@ class docParser(xml.sax.handler.ContentHandler):
             self.function_return = None
             self.function_file = None
             self.function_module= None
-            if attrs.has_key('name'):
+            if 'name' in attrs.keys():
                 self.function = attrs['name']
-            if attrs.has_key('file'):
+            if 'file' in attrs.keys():
                 self.function_file = attrs['file']
-            if attrs.has_key('module'):
+            if 'module' in attrs.keys():
                 self.function_module= attrs['module']
         elif tag == 'cond':
             self._data = []
@@ -91,24 +91,24 @@ class docParser(xml.sax.handler.ContentHandler):
                 self.function_arg_name = None
                 self.function_arg_type = None
                 self.function_arg_info = None
-                if attrs.has_key('name'):
+                if 'name' in attrs.keys():
                     self.function_arg_name = attrs['name']
                     if self.function_arg_name == 'from':
                         self.function_arg_name = 'frm'
-                if attrs.has_key('type'):
+                if 'type' in attrs.keys():
                     self.function_arg_type = attrs['type']
-                if attrs.has_key('info'):
+                if 'info' in attrs.keys():
                     self.function_arg_info = attrs['info']
         elif tag == 'return':
             if self.in_function == 1:
                 self.function_return_type = None
                 self.function_return_info = None
                 self.function_return_field = None
-                if attrs.has_key('type'):
+                if 'type' in attrs.keys():
                     self.function_return_type = attrs['type']
-                if attrs.has_key('info'):
+                if 'info' in attrs.keys():
                     self.function_return_info = attrs['info']
-                if attrs.has_key('field'):
+                if 'field' in attrs.keys():
                     self.function_return_field = attrs['field']
         elif tag == 'enum':
             # enums come from header files, hence virterror.h
@@ -123,7 +123,7 @@ class docParser(xml.sax.handler.ContentHandler):
 
     def end(self, tag):
         if debug:
-            print "end %s" % tag
+            print("end %s" % tag)
         if tag == 'function':
             # fuctions come from source files, hence 'virerror.c'
             if self.function is not None:
@@ -207,7 +207,7 @@ def lxc_function(name, desc, ret, args, file, module, cond):
     lxc_functions[name] = (desc, ret, args, file, module, cond)
 
 def enum(type, name, value):
-    if not enums.has_key(type):
+    if type not in enums:
         enums[type] = {}
     if value == 'VIR_TYPED_PARAM_INT':
         value = 1
@@ -233,14 +233,14 @@ def enum(type, name, value):
         enums[type][name] = value
 
 def lxc_enum(type, name, value):
-    if not lxc_enums.has_key(type):
+    if type not in lxc_enums:
         lxc_enums[type] = {}
     if onlyOverrides and name not in lxc_enums[type]:
         return
     lxc_enums[type][name] = value
 
 def qemu_enum(type, name, value):
-    if not qemu_enums.has_key(type):
+    if type not in qemu_enums:
         qemu_enums[type] = {}
     if onlyOverrides and name not in qemu_enums[type]:
         return
@@ -491,6 +491,8 @@ skip_function = (
     'virConnectDomainEventDeregister', # overridden in virConnect.py
     'virConnectDomainEventRegisterAny',   # overridden in virConnect.py
     'virConnectDomainEventDeregisterAny', # overridden in virConnect.py
+    'virConnectNetworkEventRegisterAny',   # overridden in virConnect.py
+    'virConnectNetworkEventDeregisterAny', # overridden in virConnect.py
     'virSaveLastError', # We have our own python error wrapper
     'virFreeError', # Only needed if we use virSaveLastError
     'virConnectListAllDomains', # overridden in virConnect.py
@@ -600,10 +602,10 @@ def print_function_wrapper(module, name, output, export, include):
         if module == "libvirt-qemu":
             (desc, ret, args, file, mod, cond) = qemu_functions[name]
     except:
-        print "failed to get function %s infos" % name
+        print("failed to get function %s infos" % name)
         return
 
-    if skipped_modules.has_key(module):
+    if module in skipped_modules:
         return 0
 
     if module == "libvirt":
@@ -637,7 +639,7 @@ def print_function_wrapper(module, name, output, export, include):
         if arg[1][0:6] == "const ":
             arg[1] = arg[1][6:]
         c_args = c_args + "    %s %s;\n" % (arg[1], arg[0])
-        if py_types.has_key(arg[1]):
+        if arg[1] in py_types:
             (f, t, n, c) = py_types[arg[1]]
             if (f == 'z') and (name in foreign_encoding_args) and (num_bufs == 0):
                 f = 't#'
@@ -659,9 +661,9 @@ def print_function_wrapper(module, name, output, export, include):
                 c_call = c_call + ", "
             c_call = c_call + "%s" % (arg[0])
         else:
-            if skipped_types.has_key(arg[1]):
+            if arg[1] in skipped_types:
                 return 0
-            if unknown_types.has_key(arg[1]):
+            if arg[1] in unknown_types:
                 lst = unknown_types[arg[1]]
                 lst.append(name)
             else:
@@ -683,7 +685,7 @@ def print_function_wrapper(module, name, output, export, include):
         else:
             c_call = "\n    %s(%s);\n" % (name, c_call)
         ret_convert = "    Py_INCREF(Py_None);\n    return Py_None;\n"
-    elif py_types.has_key(ret[0]):
+    elif ret[0] in py_types:
         (f, t, n, c) = py_types[ret[0]]
         c_return = "    %s c_retval;\n" % (ret[0])
         if file == "python_accessor" and ret[2] is not None:
@@ -692,16 +694,16 @@ def print_function_wrapper(module, name, output, export, include):
             c_call = "\n    c_retval = %s(%s);\n" % (name, c_call)
         ret_convert = "    py_retval = libvirt_%sWrap((%s) c_retval);\n" % (n,c)
         ret_convert = ret_convert + "    return py_retval;\n"
-    elif py_return_types.has_key(ret[0]):
+    elif ret[0] in py_return_types:
         (f, t, n, c) = py_return_types[ret[0]]
         c_return = "    %s c_retval;\n" % (ret[0])
         c_call = "\n    c_retval = %s(%s);\n" % (name, c_call)
         ret_convert = "    py_retval = libvirt_%sWrap((%s) c_retval);\n" % (n,c)
         ret_convert = ret_convert + "    return py_retval;\n"
     else:
-        if skipped_types.has_key(ret[0]):
+        if ret[0] in skipped_types:
             return 0
-        if unknown_types.has_key(ret[0]):
+        if ret[0] in unknown_types:
             lst = unknown_types[ret[0]]
             lst.append(name)
         else:
@@ -794,7 +796,7 @@ def buildStubs(module, api_xml):
     global onlyOverrides
 
     if module not in ["libvirt", "libvirt-qemu", "libvirt-lxc"]:
-        print "ERROR: Unknown module type: %s" % module
+        print("ERROR: Unknown module type: %s" % module)
         return None
 
     if module == "libvirt":
@@ -818,13 +820,14 @@ def buildStubs(module, api_xml):
         (parser, target)  = getparser()
         parser.feed(data)
         parser.close()
-    except IOError, msg:
-        print file, ":", msg
+    except IOError:
+        msg = sys.exc_info()[1]
+        print(file, ":", msg)
         sys.exit(1)
 
-    n = len(funcs.keys())
+    n = len(list(funcs.keys()))
     if not quiet:
-        print "Found %d functions in %s" % ((n), api_xml)
+        print("Found %d functions in %s" % ((n), api_xml))
 
     override_api_xml = "%s-override-api.xml" % module
     py_types['pythonObject'] = ('O', "pythonObject", "pythonObject", "pythonObject")
@@ -837,13 +840,14 @@ def buildStubs(module, api_xml):
         (parser, target)  = getparser()
         parser.feed(data)
         parser.close()
-    except IOError, msg:
-        print file, ":", msg
+    except IOError:
+        msg = sys.exc_info()[1]
+        print(file, ":", msg)
 
     if not quiet:
         # XXX: This is not right, same function already in @functions
         # will be overwritten.
-        print "Found %d functions in %s" % ((len(funcs.keys()) - n), override_api_xml)
+        print("Found %d functions in %s" % ((len(list(funcs.keys())) - n), override_api_xml))
     nb_wrap = 0
     failed = 0
     skipped = 0
@@ -865,7 +869,10 @@ def buildStubs(module, api_xml):
     wrapper.write("#include \"typewrappers.h\"\n")
     wrapper.write("#include \"build/" + module + ".h\"\n\n")
 
-    for function in funcs.keys():
+    funcnames = list(funcs.keys())
+    if funcnames is not None:
+        funcnames.sort()
+    for function in funcnames:
         # Skip the functions which are not for the module
         ret = print_function_wrapper(module, function, wrapper, export, include)
         if ret < 0:
@@ -883,15 +890,15 @@ def buildStubs(module, api_xml):
     wrapper.close()
 
     if not quiet:
-        print "Generated %d wrapper functions" % nb_wrap
+        print("Generated %d wrapper functions" % nb_wrap)
 
     if unknown_types:
-        print "Missing type converters: "
-        for type in unknown_types.keys():
-            print "%s:%d " % (type, len(unknown_types[type])),
+        print("Missing type converters: ")
+        for type in list(unknown_types.keys()):
+            print("%s:%d " % (type, len(unknown_types[type])))
 
     for f in funcs_failed:
-        print "ERROR: failed %s" % f
+        print("ERROR: failed %s" % f)
 
     if failed > 0:
         return -1
@@ -1021,139 +1028,139 @@ def nameFixup(name, classe, type, file):
     l = len(classe)
     if name[0:l] == listname:
         func = name[l:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:16] == "virNetworkDefine":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:19] == "virNetworkCreateXML":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:16] == "virNetworkLookup":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:18] == "virInterfaceDefine":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:21] == "virInterfaceCreateXML":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:18] == "virInterfaceLookup":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:15] == "virSecretDefine":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:15] == "virSecretLookup":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:17] == "virNWFilterDefine":
         func = name[3:]
-        func = string.lower(func[0:3]) + func[3:]
+        func = func[0:3].lower() + func[3:]
     elif name[0:17] == "virNWFilterLookup":
         func = name[3:]
-        func = string.lower(func[0:3]) + func[3:]
+        func = func[0:3].lower() + func[3:]
     elif name[0:20] == "virStoragePoolDefine":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:23] == "virStoragePoolCreateXML":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:20] == "virStoragePoolLookup":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:19] == "virStorageVolDefine":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:19] == "virStorageVolLookup":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:20] == "virDomainGetCPUStats":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:12] == "virDomainGet":
         func = name[12:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:29] == "virDomainSnapshotLookupByName":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:26] == "virDomainSnapshotListNames":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:28] == "virDomainSnapshotNumChildren":
         func = name[17:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:20] == "virDomainSnapshotNum":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:26] == "virDomainSnapshotCreateXML":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:24] == "virDomainSnapshotCurrent":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:17] == "virDomainSnapshot":
         func = name[17:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:9] == "virDomain":
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:13] == "virNetworkGet":
         func = name[13:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:10] == "virNetwork":
         func = name[10:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:15] == "virInterfaceGet":
         func = name[15:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:12] == "virInterface":
         func = name[12:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:12] == 'virSecretGet':
         func = name[12:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:9] == 'virSecret':
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:14] == 'virNWFilterGet':
         func = name[14:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:11] == 'virNWFilter':
         func = name[11:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:12] == 'virStreamNew':
         func = "newStream"
     elif name[0:9] == 'virStream':
         func = name[9:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:17] == "virStoragePoolGet":
         func = name[17:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:14] == "virStoragePool":
         func = name[14:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:16] == "virStorageVolGet":
         func = name[16:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:13] == "virStorageVol":
         func = name[13:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:13] == "virNodeDevice":
         if name[13:16] == "Get":
-            func = string.lower(name[16]) + name[17:]
+            func = name[16].lower() + name[17:]
         elif name[13:19] == "Lookup" or name[13:19] == "Create":
-            func = string.lower(name[3]) + name[4:]
+            func = name[3].lower() + name[4:]
         else:
-            func = string.lower(name[13]) + name[14:]
+            func = name[13].lower() + name[14:]
     elif name[0:7] == "virNode":
         func = name[7:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:10] == "virConnect":
         func = name[10:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     elif name[0:3] == "xml":
         func = name[3:]
-        func = string.lower(func[0:1]) + func[1:]
+        func = func[0:1].lower() + func[1:]
     else:
         func = name
     if func == "iD":
@@ -1172,23 +1179,9 @@ def nameFixup(name, classe, type, file):
     return func
 
 
-def functionCompare(info1, info2):
-    (index1, func1, name1, ret1, args1, file1, mod1) = info1
-    (index2, func2, name2, ret2, args2, file2, mod2) = info2
-    if file1 == file2:
-        if func1 < func2:
-            return -1
-        if func1 > func2:
-            return 1
-    if file1 == "python_accessor":
-        return -1
-    if file2 == "python_accessor":
-        return 1
-    if file1 < file2:
-        return -1
-    if file1 > file2:
-        return 1
-    return 0
+def functionSortKey(info):
+    (index, func, name, ret, args, filename, mod) = info
+    return func, filename
 
 def writeDoc(module, name, args, indent, output):
      if module == "libvirt":
@@ -1200,15 +1193,15 @@ def writeDoc(module, name, args, indent, output):
      if funcs[name][0] is None or funcs[name][0] == "":
          return
      val = funcs[name][0]
-     val = string.replace(val, "NULL", "None")
+     val = val.replace("NULL", "None")
      output.write(indent)
      output.write('"""')
-     i = string.find(val, "\n")
+     i = val.find("\n")
      while i >= 0:
          str = val[0:i+1]
          val = val[i+1:]
          output.write(str)
-         i = string.find(val, "\n")
+         i = val.find("\n")
          output.write(indent)
      output.write(val)
      output.write(' """\n')
@@ -1229,10 +1222,10 @@ def buildWrappers(module):
     global functions_noexcept
 
     if not module == "libvirt":
-        print "ERROR: Unknown module type: %s" % module
+        print("ERROR: Unknown module type: %s" % module)
         return None
 
-    for type in classes_type.keys():
+    for type in list(classes_type.keys()):
         function_classes[classes_type[type][2]] = []
 
     #
@@ -1246,23 +1239,23 @@ def buildWrappers(module):
     for classe in primary_classes:
         classes_list.append(classe)
         classes_processed[classe] = ()
-        for type in classes_type.keys():
+        for type in list(classes_type.keys()):
             tinfo = classes_type[type]
             if tinfo[2] == classe:
                 ctypes.append(type)
                 ctypes_processed[type] = ()
-    for type in classes_type.keys():
-        if ctypes_processed.has_key(type):
+    for type in list(classes_type.keys()):
+        if type in ctypes_processed:
             continue
         tinfo = classes_type[type]
-        if not classes_processed.has_key(tinfo[2]):
+        if tinfo[2] not in classes_processed:
             classes_list.append(tinfo[2])
             classes_processed[tinfo[2]] = ()
 
         ctypes.append(type)
         ctypes_processed[type] = ()
 
-    for name in functions.keys():
+    for name in list(functions.keys()):
         found = 0
         (desc, ret, args, file, mod, cond) = functions[name]
         for type in ctypes:
@@ -1317,9 +1310,9 @@ def buildWrappers(module):
     if extra is not None:
         extra.close()
 
-    if function_classes.has_key("None"):
+    if "None" in function_classes:
         flist = function_classes["None"]
-        flist.sort(functionCompare)
+        flist.sort(key=functionSortKey)
         oldfile = ""
         for info in flist:
             (index, func, name, ret, args, file, mod) = info
@@ -1342,7 +1335,7 @@ def buildWrappers(module):
             writeDoc(module, name, args, '    ', classes)
 
             for arg in args:
-                if classes_type.has_key(arg[1]):
+                if arg[1] in classes_type:
                     classes.write("    if %s is None: %s__o = None\n" %
                                   (arg[0], arg[0]))
                     classes.write("    else: %s__o = %s%s\n" %
@@ -1357,17 +1350,17 @@ def buildWrappers(module):
                 if n != 0:
                     classes.write(", ")
                 classes.write("%s" % arg[0])
-                if classes_type.has_key(arg[1]):
+                if arg[1] in classes_type:
                     classes.write("__o")
                 n = n + 1
             classes.write(")\n")
 
             if ret[0] != "void":
-                if classes_type.has_key(ret[0]):
+                if ret[0] in classes_type:
                     #
                     # Raise an exception
                     #
-                    if functions_noexcept.has_key(name):
+                    if name in functions_noexcept:
                         classes.write("    if ret is None:return None\n")
                     else:
                         classes.write(
@@ -1382,8 +1375,8 @@ def buildWrappers(module):
                 # several things that we can do, depending on the
                 # contents of functions_int_*:
                 elif is_integral_type (ret[0]):
-                    if not functions_noexcept.has_key (name):
-                        if functions_int_exception_test.has_key (name):
+                    if name not in functions_noexcept:
+                        if name in functions_int_exception_test:
                             test = functions_int_exception_test[name]
                         else:
                             test = functions_int_default_test
@@ -1393,8 +1386,8 @@ def buildWrappers(module):
                     classes.write("    return ret\n")
 
                 elif is_python_noninteger_type (ret[0]):
-                    if not functions_noexcept.has_key (name):
-                        if functions_list_exception_test.has_key (name):
+                    if name not in functions_noexcept:
+                        if name in functions_list_exception_test:
                             test = functions_list_exception_test[name]
                         else:
                             test = functions_list_default_test
@@ -1412,11 +1405,11 @@ def buildWrappers(module):
         if classname == "None":
             pass
         else:
-            if classes_ancestor.has_key(classname):
+            if classname in classes_ancestor:
                 classes.write("class %s(%s):\n" % (classname,
                               classes_ancestor[classname]))
                 classes.write("    def __init__(self, _obj=None):\n")
-                if reference_keepers.has_key(classname):
+                if classname in reference_keepers:
                     rlist = reference_keepers[classname]
                     for ref in rlist:
                         classes.write("        self.%s = None\n" % ref[1])
@@ -1433,9 +1426,9 @@ def buildWrappers(module):
                     classes.write("    def __init__(self, dom, _obj=None):\n")
                 else:
                     classes.write("    def __init__(self, _obj=None):\n")
-                if reference_keepers.has_key(classname):
-                    list = reference_keepers[classname]
-                    for ref in list:
+                if classname in reference_keepers:
+                    rlist = reference_keepers[classname]
+                    for ref in rlist:
                         classes.write("        self.%s = None\n" % ref[1])
                 if classname in [ "virDomain", "virNetwork", "virInterface",
                                   "virNodeDevice", "virSecret", "virStream",
@@ -1450,7 +1443,7 @@ def buildWrappers(module):
                     classes.write("        self._conn = dom.connect()\n")
                 classes.write("        self._o = _obj\n\n")
             destruct=None
-            if classes_destructors.has_key(classname):
+            if classname in classes_destructors:
                 classes.write("    def __del__(self):\n")
                 classes.write("        if self._o is not None:\n")
                 classes.write("            libvirtmod.%s(self._o)\n" %
@@ -1458,17 +1451,17 @@ def buildWrappers(module):
                 classes.write("        self._o = None\n\n")
                 destruct=classes_destructors[classname]
 
-            if not class_skip_connect_impl.has_key(classname):
+            if classname not in class_skip_connect_impl:
                 # Build python safe 'connect' method
                 classes.write("    def connect(self):\n")
                 classes.write("        return self._conn\n\n")
 
-            if class_domain_impl.has_key(classname):
+            if classname in class_domain_impl:
                 classes.write("    def domain(self):\n")
                 classes.write("        return self._dom\n\n")
 
             flist = function_classes[classname]
-            flist.sort(functionCompare)
+            flist.sort(key=functionSortKey)
             oldfile = ""
             for info in flist:
                 (index, func, name, ret, args, file, mod) = info
@@ -1502,7 +1495,7 @@ def buildWrappers(module):
                 writeDoc(module, name, args, '        ', classes)
                 n = 0
                 for arg in args:
-                    if classes_type.has_key(arg[1]):
+                    if arg[1] in classes_type:
                         if n != index:
                             classes.write("        if %s is None: %s__o = None\n" %
                                           (arg[0], arg[0]))
@@ -1520,11 +1513,11 @@ def buildWrappers(module):
                         classes.write(", ")
                     if n != index:
                         classes.write("%s" % arg[0])
-                        if classes_type.has_key(arg[1]):
+                        if arg[1] in classes_type:
                             classes.write("__o")
                     else:
                         classes.write("self")
-                        if classes_type.has_key(arg[1]):
+                        if arg[1] in classes_type:
                             classes.write(classes_type[arg[1]][0])
                     n = n + 1
                 classes.write(")\n")
@@ -1534,11 +1527,11 @@ def buildWrappers(module):
 
                 # For functions returning object types:
                 if ret[0] != "void":
-                    if classes_type.has_key(ret[0]):
+                    if ret[0] in classes_type:
                         #
                         # Raise an exception
                         #
-                        if functions_noexcept.has_key(name):
+                        if name in functions_noexcept:
                             classes.write(
                                 "        if ret is None:return None\n")
                         else:
@@ -1588,15 +1581,15 @@ def buildWrappers(module):
                         # See reference_keepers for the list
                         #
                         tclass = classes_type[ret[0]][2]
-                        if reference_keepers.has_key(tclass):
-                            list = reference_keepers[tclass]
-                            for pref in list:
+                        if tclass in reference_keepers:
+                            rlist = reference_keepers[tclass]
+                            for pref in rlist:
                                 if pref[0] == classname:
                                     classes.write("        __tmp.%s = self\n" %
                                                   pref[1])
 
                         # Post-processing - just before we return.
-                        if function_post.has_key(name):
+                        if name in function_post:
                             classes.write("        %s\n" %
                                           (function_post[name]))
 
@@ -1604,16 +1597,16 @@ def buildWrappers(module):
                         # return the class
                         #
                         classes.write("        return __tmp\n")
-                    elif converter_type.has_key(ret[0]):
+                    elif ret[0] in converter_type:
                         #
                         # Raise an exception
                         #
-                        if functions_noexcept.has_key(name):
+                        if name in functions_noexcept:
                             classes.write(
                                 "        if ret is None:return None")
 
                         # Post-processing - just before we return.
-                        if function_post.has_key(name):
+                        if name in function_post:
                             classes.write("        %s\n" %
                                           (function_post[name]))
 
@@ -1625,8 +1618,8 @@ def buildWrappers(module):
                     # are several things that we can do, depending on
                     # the contents of functions_int_*:
                     elif is_integral_type (ret[0]):
-                        if not functions_noexcept.has_key (name):
-                            if functions_int_exception_test.has_key (name):
+                        if name not in functions_noexcept:
+                            if name in functions_int_exception_test:
                                 test = functions_int_exception_test[name]
                             else:
                                 test = functions_int_default_test
@@ -1660,15 +1653,15 @@ def buildWrappers(module):
                                                ("ret", name))
 
                         # Post-processing - just before we return.
-                        if function_post.has_key(name):
+                        if name in function_post:
                             classes.write("        %s\n" %
                                           (function_post[name]))
 
                         classes.write ("        return ret\n")
 
                     elif is_python_noninteger_type (ret[0]):
-                        if not functions_noexcept.has_key (name):
-                            if functions_list_exception_test.has_key (name):
+                        if name not in functions_noexcept:
+                            if name in functions_list_exception_test:
                                 test = functions_list_exception_test[name]
                             else:
                                 test = functions_list_default_test
@@ -1702,7 +1695,7 @@ def buildWrappers(module):
                                                ("ret", name))
 
                         # Post-processing - just before we return.
-                        if function_post.has_key(name):
+                        if name in function_post:
                             classes.write("        %s\n" %
                                           (function_post[name]))
 
@@ -1710,7 +1703,7 @@ def buildWrappers(module):
 
                     else:
                         # Post-processing - just before we return.
-                        if function_post.has_key(name):
+                        if name in function_post:
                             classes.write("        %s\n" %
                                           (function_post[name]))
 
@@ -1772,10 +1765,13 @@ def buildWrappers(module):
     #
     # Generate enum constants
     #
-    for type,enum in enums.items():
+    enumvals = list(enums.items())
+    if enumvals is not None:
+        enumvals.sort(key=lambda x: x[0])
+    for type,enum in enumvals:
         classes.write("# %s\n" % type)
-        items = enum.items()
-        items.sort(lambda i1,i2: cmp(long(i1[1]),long(i2[1])))
+        items = list(enum.items())
+        items.sort(key=lambda i: int(i[1]))
         for name,value in items:
             classes.write("%s = %s\n" % (name,value))
         classes.write("\n")
@@ -1786,7 +1782,7 @@ def qemuBuildWrappers(module):
     global qemu_functions
 
     if not module == "libvirt-qemu":
-        print "ERROR: only libvirt-qemu is supported"
+        print("ERROR: only libvirt-qemu is supported")
         return None
 
     extra_file = "%s-override.py" % module
@@ -1820,10 +1816,12 @@ def qemuBuildWrappers(module):
 
     fd.write("try:\n")
     fd.write("    import libvirtmod_qemu\n")
-    fd.write("except ImportError, lib_e:\n")
+    fd.write("except ImportError:\n")
+    fd.write("    lib_e = sys.exc_info()[1]\n")
     fd.write("    try:\n")
     fd.write("        import cygvirtmod_qemu as libvirtmod_qemu\n")
-    fd.write("    except ImportError, cyg_e:\n")
+    fd.write("    except ImportError:\n")
+    fd.write("        cyg_e = sys.exc_info()[1]\n")
     fd.write("        if str(cyg_e).count(\"No module named\"):\n")
     fd.write("            raise lib_e\n\n")
 
@@ -1832,7 +1830,7 @@ def qemuBuildWrappers(module):
     #
     # Generate functions directly, no classes
     #
-    for name in qemu_functions.keys():
+    for name in list(qemu_functions.keys()):
         func = nameFixup(name, 'None', None, None)
         (desc, ret, args, file, mod, cond) = qemu_functions[name]
         fd.write("def %s(" % func)
@@ -1882,10 +1880,10 @@ def qemuBuildWrappers(module):
     #
     # Generate enum constants
     #
-    for type,enum in qemu_enums.items():
+    for type,enum in list(qemu_enums.items()):
         fd.write("# %s\n" % type)
-        items = enum.items()
-        items.sort(lambda i1,i2: cmp(long(i1[1]),long(i2[1])))
+        items = list(enum.items())
+        items.sort(key=lambda i: int(i[1]))
         for name,value in items:
             fd.write("%s = %s\n" % (name,value))
         fd.write("\n")
@@ -1897,7 +1895,7 @@ def lxcBuildWrappers(module):
     global lxc_functions
 
     if not module == "libvirt-lxc":
-        print "ERROR: only libvirt-lxc is supported"
+        print("ERROR: only libvirt-lxc is supported")
         return None
 
     extra_file = "%s-override.py" % module
@@ -1931,10 +1929,12 @@ def lxcBuildWrappers(module):
 
     fd.write("try:\n")
     fd.write("    import libvirtmod_lxc\n")
-    fd.write("except ImportError, lib_e:\n")
+    fd.write("except ImportError:\n")
+    fd.write("    lib_e = sys.exc_info()[1]\n")
     fd.write("    try:\n")
     fd.write("        import cygvirtmod_lxc as libvirtmod_lxc\n")
-    fd.write("    except ImportError, cyg_e:\n")
+    fd.write("    except ImportError:\n")
+    fd.write("        cyg_e = sys.exc_info()[1]\n")
     fd.write("        if str(cyg_e).count(\"No module named\"):\n")
     fd.write("            raise lib_e\n\n")
 
@@ -1943,7 +1943,7 @@ def lxcBuildWrappers(module):
     #
     # Generate functions directly, no classes
     #
-    for name in lxc_functions.keys():
+    for name in list(lxc_functions.keys()):
         func = nameFixup(name, 'None', None, None)
         (desc, ret, args, file, mod, cond) = lxc_functions[name]
         fd.write("def %s(" % func)
@@ -1993,10 +1993,10 @@ def lxcBuildWrappers(module):
     #
     # Generate enum constants
     #
-    for type,enum in lxc_enums.items():
+    for type,enum in list(lxc_enums.items()):
         fd.write("# %s\n" % type)
-        items = enum.items()
-        items.sort(lambda i1,i2: cmp(long(i1[1]),long(i2[1])))
+        items = list(enum.items())
+        items.sort(key=lambda i: int(i[1]))
         for name,value in items:
             fd.write("%s = %s\n" % (name,value))
         fd.write("\n")
@@ -2018,7 +2018,7 @@ elif sys.argv[1] == "libvirt-lxc":
 elif sys.argv[1] == "libvirt-qemu":
     qemuBuildWrappers(sys.argv[1])
 else:
-    print "ERROR: unknown module %s" % sys.argv[1]
+    print("ERROR: unknown module %s" % sys.argv[1])
     sys.exit(1)
 
 sys.exit(0)
